// Copyright (C) 2009 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// Assessing the quality of the Normal distribution function
// References
//   Yalta, A. T. 2008. The accuracy of statistical distributions in MicrosoftÂ®Excel 2007. Comput. Stat. Data Anal. 52, 10 (Jun. 2008), 4579-4586. DOI= http://dx.doi.org/10.1016/j.csda.2008.03.005 
//   Computation of Statistical Distributions (ELV), Leo KnÃ¼sel 
//

// We need the Poisson distribution function.
// There is no one in Scilab, so we write our own.

// dfpoisson --
//   Poisson distribution function
//   Not a naive implementation :
//   avoid overflows and vectorize.
function p = dfpoisson ( k , lambda )
    p = exp ( - lambda )
    for i = 1 : k
        p = p * lambda / i
    end
endfunction

/////////////////////////////////////////////////////////////////////
// From Stixbox 1.2.5

// This implementation is NOT accurate.
// Example : 
//   p = dhypg ( 200 , 515 , 500 , 1030 )
// Result :
///  p = 0.0  // Expected result = 1.65570E-ˆ’10
//
// Reference 
//   "An Accurate Computation of the Hypergeometric Distribution Function", 
//   Trong Wu, ACM Transactions on Mathematical Software, Vol. 19, No. 1, March 1993, Pages 33-43 
function [p]=dhypg(k,n,K,N)
    p=[];
    // The hypergeometric probability function
    //
    //      p = dhypg(k,n,K,N)
    //	Input	k,n,K,N nonnegative integers such that 
    //			n<= N, K<= N and n-k<= N-K	
    //			N is the total number of elements of the population
    //			n is the number of elements which are randomly
    //			  extracted
    //			K is the number of elements of the population which 
    //			  have the studied property
    //
    //	Output  p	probability Prob(X=k) where X is an hypergeometric
    //			random variable with parameters n,K,N
    //       Anders Holtsberg, 18-11-93
    //       Copyright (c) Anders Holtsberg
    //   last update: dec 2001 (jpc)

    if or(round(n)~=n | round(N)~=N |round(k)~=k |round(K)~=K)
        error('dhypg: requires integer arguments');
    end

    if or(n>N | K>N | K<0) then
        error('Incompatible input arguments');
    end

    z = k < 0 | (n-k) < 0 | k > K | (n-k)>(N-K);
    //  mtlb_find(~z) may be replaced by
    //  find(~z)' if ~z is not a row vector
    I = mtlb_find(~z);
    if mtlb_length(k)>1 then
        // mtlb_e(k,I) may be replaced by k(I) if k is a vector.
        k = mtlb_e(k,I);
    end
    if mtlb_length(K)>1 then
        // mtlb_e(K,I) may be replaced by K(I) if K is a vector.
        K = mtlb_e(K,I);
    end
    if mtlb_length(n)>1 then
        // mtlb_e(n,I) may be replaced by n(I) if n is a vector.
        n = mtlb_e(n,I);
    end
    if mtlb_length(N)>1 then
        // mtlb_e(N,I) may be replaced by N(I) if N is a vector.
        N = mtlb_e(N,I);
    end
    pp = bincoef(k,K) .* bincoef(n-k,N-K) ./ bincoef(n,N);
    p = bool2s(z)*0;
    if(size(p,1) == 1) then 
        p(I) = pp(:)'; 
    else 
        p(I)=pp(:);
    end;
endfunction

function [k]=bincoef(n,N)
    k=[];
    //  Binomial coefficients
    //         k = bincoef(n,N)
    //       Anders Holtsberg, 13-05-94
    //       Copyright (c) Anders Holtsberg
    // 	Input	n,N 	scalar or matrix with common size, such that 0<=n<=N
    //
    //	Output	k	matrix of binomial coefficients
    //			Gamma(N+1)/(Gamma(n+1)Gamma(N-n+1))
    // last update: dec 2001 (jpc)
    k = exp(gammaln(N+1)-gammaln(n+1)-gammaln(N-n+1));
    if and(round(n)==n) & and(round(N)==N) then
        k = round(k)
    end
endfunction
function r=mtlb_length(a)
    if type(a)==10 then
        r=length(a)
    else
        r=max(size(a))
    end
endfunction
/////////////////////////////////////////////////////////////////////
//
///
//// HYPERGEOMETRIC_PDF evaluates the Hypergeometric PDF.
//
//  Formula:
//    PDF(X)(N,M,L) = C(M,X) * C(L-M,N-X) / C(L,N).
//
//  Definition:
//    PDF(X)(N,M,L) is the probability of drawing X white balls in a
//    single random sample of size N from a population containing
//    M white balls and a total of L balls.
//
//  Modified:
//    12 September 2004
//
//  Author:
//    John Burkardt
//    Scilab version : Michael Baudin
//
//  Parameters:
//    Input, integer X, the desired number of white balls.
//    0 <= X <= N, usually, although any value of X can be given.
//
//    Input, integer N, the number of balls selected.
//    0 <= N <= L.
//
//    Input, integer M, the number of white balls in the population.
//    0 <= M <= L.
//
//    Input, integer L, the number of balls to select from.
//    0 <= L.
//
//    Output, real PDF, the probability of exactly K white balls.
//
function pdf = hypergeometric_pdf ( x, n, m, l )
    //
    //  Special cases.
    //
    if ( x < 0 )
        pdf = 1.0;
    elseif ( n < x )
        pdf = 0.0;
    elseif ( m < x )
        pdf = 0.0;
    elseif ( l < x )
        pdf = 0.0;
    elseif ( n == 0 )
        if ( x == 0 )
            pdf = 1.0;
        else
            pdf = 0.0;
        end
    else
        c1 = binomial_coef_log (   m,   x );
        c2 = binomial_coef_log ( l-m, n-x );
        c3 = binomial_coef_log ( l,   n   );
        pdf_log = c1 + c2 - c3;
        pdf = exp ( pdf_log );
    end
endfunction
//// BINOMIAL_COEF_LOG computes the logarithm of the Binomial coefficient.
//
//  Formula:
//    CNK_LOG = LOG ( C(N,K) ) = LOG ( N! / ( K! * (N-K)! ) ).
//
//  Modified:
//    03 September 2004
//
//  Author:
//    John Burkardt
//    Scilab version : Michael Baudin
//
//  Parameters:
//    Input, integer N, K, are the values of N and K.
//    Output, real CNK_LOG, the logarithm of C(N,K).
//
function cnk_log = binomial_coef_log ( n, k )

    cnk_log = factorial_log ( n ) - factorial_log ( k ) - factorial_log ( n - k );
endfunction
//// FACTORIAL_LOG returns the logarithm of N!.
//
//  Definition:
//    N! = Product ( 1 <= I <= N ) I
//
//  Method:
//    N! = Gamma(N+1).
//
//  Modified:
//    11 September 2004
//
//  Author:
//    John Burkardt
//    Scilab version : Michael Baudin
//
//  Parameters:
//    Input, integer N, the argument of the function.
//    0 <= N.
//
//    Output, real FACTORIAL_LOG, the logarithm of N!.
//
function value = factorial_log ( n )

    if ( n < 0 )
        fprintf ( 1, '\n' );
        fprintf ( 1, 'FACTORIAL_LOG - Fatal error!\n' );
        fprintf ( 1, '  N < 0.\n' );
        error ( 'FACTORIAL_LOG - Fatal error!' );
    end
    value = 0.0;
    for i = 2 : n
        value = value + log ( i );
    end
endfunction

/////////////////////////////////////////////////////////////////////

//
// Table 6
// Check inverse of normal law
//
function checknormal ( p , expected , precision ) 
    q = 1 - p
    Mean = 0.0
    Std = 1.0
    computed = cdfnor ( "X" , Mean , Std , p , q )
    assert_checkalmostequal ( computed , expected , precision );
endfunction

checknormal ( 0.5 , 0.0 , %eps ) 
checknormal ( 1.e-1 , -1.28155 , 1.e-5 ) 
checknormal ( 1.e-2 , -2.32635 , 1.e-5 ) 
checknormal ( 1.e-3 , -3.09023 , 1.e-5 ) 
checknormal ( 1.e-4 , -3.71902 , 1.e-5 ) 
checknormal ( 1.e-5 , -4.26489 , 1.e-5 ) 
checknormal ( 1.e-6 , -4.75342 , 1.e-5 ) 
checknormal ( 1.e-7 , -5.19934 , 1.e-5 ) 
checknormal ( 1.e-15 , -7.94135 , 1.e-5 ) 
checknormal ( 1.e-16 , -8.22208 , 1.e-5 ) 
checknormal ( 1.e-100 , -21.2735 , 1.e-5 ) 
checknormal ( 1.e-197 , -29.9763 , 1.e-5 ) 
checknormal ( 1.e-198 , -30.0529 , 1.e-5 ) 
checknormal ( 1.e-300 , -37.0471 , 1.e-5 ) 

//
// Table 2
// Check Binomial distribution with parameters (k, n = 1030, p = 0.5)
//
function checkbinomial ( k , expected , precision ) 
    Pr = 0.5
    Ompr = 1 - Pr
    Xn = 1030
    [computed,Q]=cdfbin("PQ",k,Xn,Pr,Ompr)
    assert_checkalmostequal ( computed , expected , precision );
endfunction

checkbinomial ( 1 , 8.96114e-308 , 1.e-5 ) 
checkbinomial ( 2 , 4.61499e-305 , 1.e-5 ) 
checkbinomial ( 100 , 1.39413e-169 , 1.e-5 ) 
checkbinomial ( 200 , 5.45781e-92 , 1.e-5 ) 
checkbinomial ( 300 , 2.91621e-42 , 1.e-5 ) 
checkbinomial ( 390 , 3.18196e-15 , 1.e-5 ) 
checkbinomial ( 391 , 5.24099e-15 , 1.e-5 ) 
checkbinomial ( 400 , 3.89735e-13 , 1.e-5 ) 
checkbinomial ( 410 , 3.19438e-11 , 1.e-5 ) 
checkbinomial ( 420 , 1.76037e-09 , 1.e-5 ) 
checkbinomial ( 500 , 1.83106e-01 , 1.e-5 ) 
checkbinomial ( 550 , 9.86550e-01 , 1.e-5 ) 
checkbinomial ( 575 , 9.99920e-01 , 1.e-5 ) 
checkbinomial ( 589 , 9.99998e-01 , 1.e-5 ) 

// Table 3
// Hypergeometric distribution with parameters (k, N = 1030, M = 515, n = 500)
function checkHypergeometric ( k , expected , precision ) 
    N = 1030
    M = 515
    n = 500
    computed = hypergeometric_pdf ( k , M , n , N )
    assert_checkalmostequal ( computed , expected , precision );
endfunction

checkHypergeometric ( 0   , 1.60137e-280 , 1.e-5 ) 
checkHypergeometric ( 100 , 7.46483e-83  , 1.e-5 ) 
checkHypergeometric ( 187 , 1.53541e-15  , 1.e-5 ) 
checkHypergeometric ( 188 , 4.13038e-15  , 1.e-5 ) 
checkHypergeometric ( 200 , 1.65570e-10  , 1.e-5 ) 
checkHypergeometric ( 300 , 1.65570e-10  , 1.e-5 ) 
checkHypergeometric ( 312 , 4.13038e-15  , 1.e-5 ) 
checkHypergeometric ( 313 , 1.53541e-15  , 1.e-5 ) 
checkHypergeometric ( 400 , 7.46483e-83  , 1.e-5 ) 
checkHypergeometric ( 500 , 1.60137e-280 , 1.e-5 ) 


//
// Table 4
// Check Poisson distribution with parameters (lambda, k, Sigma)
// If Sigma = 0, the distribution function is to be computed.
// If Sigma = 1, the cumulated distribution function is to be computed.
//
function checkPoisson ( k, lambda, Sigma , expected , precision ) 
    if ( Sigma == 0 ) then
        // There is no such function in Scilab : provide our own
        computed = dfpoisson ( k , lambda )
    else
        [computed,Q]=cdfpoi("PQ",k,lambda)
    end
    assert_checkalmostequal ( computed , expected , precision );
endfunction

checkPoisson ( 0   ,   200   , 0 , 1.38390e-87 , 1.e-5)
checkPoisson ( 103 ,   200   , 0 , 1.41720e-14 , 1.e-5)
checkPoisson ( 104 ,   200   , 0 , 2.72538e-14 , 1.e-5)
checkPoisson ( 133 ,   200   , 0 , 1.01322e-07 , 1.e-5)
checkPoisson ( 134 ,   200   , 0 , 1.51227e-07 , 1.e-5)
checkPoisson ( 200 ,   200   , 0 , 2.81977e-02 , 1.e-5)
checkPoisson ( 314 ,   200   , 0 , 2.23568e-14 , 1.e-5)
checkPoisson ( 315 ,   200   , 0 , 1.41948e-14 , 1.e-5)
checkPoisson ( 400 ,   200   , 0 , 5.58069e-36 , 1.e-5)
checkPoisson ( 900 ,   200   , 0 , 1.73230e-286 , 1.e-5)
checkPoisson ( 1e+03 , 1e+03 , 1 , 0.508409 , 1.e-5)
checkPoisson ( 1e+05 , 1e+05 , 1 , 0.500841 , 1.e-5)
checkPoisson ( 1e+07 , 1e+07 , 1 , 0.500084 , 1.e-5)
checkPoisson ( 1e+09 , 1e+09 , 1 , 0.500008 , 1.e-5)
//
// Table 5
// Check Gamma distribution with parameters (x, alpha, beta = 1, Sigma = 1)
//
function checkGamma ( x , alpha , expected , precision ) 
    Shape = alpha
    Scale = 1.0
    [computed,Q]=cdfgam("PQ",x,Shape,Scale)
    assert_checkalmostequal ( computed , expected , precision );
endfunction

checkGamma ( 0.1 , 0.1 , 0.827552 , 1.e-5)
checkGamma ( 0.2 , 0.1 , 0.879420 , 1.e-5)
checkGamma ( 0.2 , 0.2 , 0.764435 , 1.e-5)
checkGamma ( 0.3 , 0.2 , 0.816527 , 1.e-5)
checkGamma ( 0.3 , 0.3 , 0.726957 , 1.e-5)
checkGamma ( 0.4 , 0.3 , 0.776381 , 1.e-5)
checkGamma ( 0.4 , 0.4 , 0.701441 , 1.e-5)
checkGamma ( 0.5 , 0.4 , 0.748019 , 1.e-5)
checkGamma ( 0.5 , 0.5 , 0.682689 , 1.e-5)
checkGamma ( 0.6 , 0.5 , 0.726678 , 1.e-5)

