// Copyright (C) 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_exprnd ( varargin )
    // Exponential random numbers
    //
    // Calling Sequence
    //   R = distfun_exprnd ( mu )
    //   R = distfun_exprnd ( mu , v )
    //   R = distfun_exprnd ( mu , m , n )
    //
    // Parameters
    //   mu : a matrix of doubles, the average. Must be positive.
    //   v : a 1x2 or 2x1 matrix of doubles, the size of R
    //   v(1) : the number of rows of R
    //   v(2) : the number of columns of R
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers.
    //
    // Description
    //   Generates random variables by inversion of the Beta cumulated probability distribution function.
    //
    //   Any scalar input argument is expanded to a matrix of doubles of the same size as the other input arguments.
    //
    //   As a side effect, it modifies the internal seed of the grand function.
    // 
    // Notice that mu, the average, is the inverse of the rate. 
    // Other computing languages (including R), use 1/mu as the 
    // parameter of the exponential distribution.
    //
    // Examples
    //
    // // Set the seed so as to always get the same results.
    // grand("setsd",1);
    //
    // // Use R = distfun_exprnd ( mu )
    // distfun_exprnd(1:6)
    //
    // // Check mean and variance for R = distfun_exprnd ( mu )
    // N = 1000;
    // for mu = 1:6;
    //   computed = distfun_exprnd(mu,[1 N]);
    //   disp(mu)
    //   c = mean(computed)
    //   e = mu
    //   c = st_deviation(computed)
    //   e = mu
    // end
    //
    // // Check R = distfun_exprnd ( mu , v )
    // computed = distfun_exprnd(2,[1 5])
    // computed = distfun_exprnd(2,[3 2])
    //
    // // Check mean and variance for R = distfun_exprnd ( a , b )
    // N = 1000;
    // mu = 2;
    // computed = distfun_exprnd(mu,[1 N]);
    // c = mean(computed(1:N) )
    // e = mu
    // c = st_deviation(computed(1:N) )
    // e = mu
    //
    // // Check R = distfun_exprnd ( mu , m , n )
    // computed = distfun_exprnd([1 2 3;4 5 6],2,3)
    // computed = distfun_exprnd(2,2,3)
    // computed = distfun_exprnd(1,2,3)
    //
    // // Check mean and variance for R = distfun_exprnd ( mu )
    // N = 1000;
    // mu = 2;
    // computed = distfun_exprnd(mu,1,N);
    // c = mean(computed(1:N) )
    // e = mu
    // c = st_deviation(computed(1:N) )
    // e = mu
    //
    // // Make a plot of the actual distribution of the numbers
    // mu = 2;
    // data = distfun_exprnd(mu,1,1000);
    // histplot(10,data)
    // x = linspace(0,14,1000);
    // y = distfun_exppdf(x,mu);
    // plot(x,y)
    //
    // Authors
    //   Copyright (C) 2009-2011 - DIGITEO - Michael Baudin

    [lhs,rhs]=argn()
    apifun_checkrhs ( "distfun_exprnd" , rhs , 1:3 )
    apifun_checklhs ( "distfun_exprnd" , lhs , 0:1 )
    //
    mu = varargin(1)
    //
    // Check type
    apifun_checktype ( "distfun_exprnd" , mu , "mu" , 1 , "constant" )
    if ( rhs == 2 ) then
        v = varargin(2)
        apifun_checktype ( "distfun_exprnd" , v , "v" , 2 , "constant" )  
    end
    if ( rhs == 3 ) then
        m = varargin(2)
        n = varargin(3)
        apifun_checktype ( "distfun_exprnd" , m , "m" , 2 , "constant" )  
        apifun_checktype ( "distfun_exprnd" , n , "n" , 3 , "constant" )  
    end
    //
    // Check size
    if ( rhs == 2 ) then
        apifun_checkvector ( "distfun_exprnd" , v , "v" , 2 , 2 )
    end
    if ( rhs == 3 ) then
        apifun_checkscalar ( "distfun_exprnd" , m , "m" , 2 )
        apifun_checkscalar ( "distfun_exprnd" , n , "n" , 3 )
    end
    //
    // Check content
    if ( rhs == 2 ) then
        apifun_checkflint ( "distfun_exprnd" , v , "v" , 2 )
        apifun_checkgreq ( "distfun_exprnd" , v , "v" , 2 , 1 )
    end
    if ( rhs == 3 ) then
        apifun_checkflint ( "distfun_exprnd" , m , "m" , 2 )
        apifun_checkgreq ( "distfun_exprnd" , m , "m" , 2 , 1 )
        apifun_checkflint ( "distfun_exprnd" , n , "n" , 3 )
        apifun_checkgreq ( "distfun_exprnd" , n , "n" , 3 , 1 )
    end
    //
    mu = apifun_expandfromsize ( 1 , varargin(1:$) )
    m = size(mu,"r")
    n = size(mu,"c")
    urnd = grand ( m , n , "unf" , 0 , 1 )
    R = distfun_expinv ( urnd , mu )
endfunction

