// Copyright (C) 2009 - 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_normrnd ( varargin )
  // Normal random numbers
  //
  // Calling Sequence
  //   R = distfun_normrnd ( mu , sigma )
  //   R = distfun_normrnd ( mu , sigma , v )
  //   R = distfun_normrnd ( mu , sigma , m , n )
  //
  // Parameters
  //   mu : a 1x1 or nxm matrix of doubles, the average
  //   sigma : a 1x1 or nxm matrix of doubles, the standard deviation
  //   v : a 1x2 or 2x1 matrix of doubles, the size of R
  //   v(1) : the number of rows of R
  //   v(2) : the number of columns of R
  //   m : a 1x1 matrix of floating point integers, the number of rows of R
  //   n : a 1x1 matrix of floating point integers, the number of columns of R
  //   R: a matrix of doubles, the random numbers.
  //
  // Description
  //   Generates random variables from the Normal distribution function.
  //
  //   Any scalar input argument is expanded to a matrix of doubles of the same size as the other input arguments.
  //
  //   As a side effect, it modifies the internal seed of the grand function.
  // 
  // Examples
  // // Test both mu and sigma expanded
  // computed = distfun_normrnd(1:6,(1:6)^-1)
  //
  // // Test sigma expansion
  // computed = distfun_normrnd(1:6,2.0)
  //
  // // Test mu expansion
  // computed = distfun_normrnd(1.0,1:6)
  //
  // // Test with v
  // computed = distfun_normrnd(0,1,[3 2])
  //
  // // Test with m, n
  // computed = distfun_normrnd(0,1,3,2)
  //
  // // Make a plot of the actual distribution of the numbers
  // mu = 2;
  // sigma = 3;
  // data = distfun_normrnd(mu,sigma,1,1000);
  // histplot(10,data)
  // x = linspace(-10,10,1000);
  // y = distfun_normpdf(x,mu,sigma);
  // plot(x,y)
  //
  // Authors
  //   Copyright (C) 2009 - 2011 - DIGITEO - Michael Baudin

  [lhs,rhs]=argn()
  apifun_checkrhs ( "distfun_normrnd" , rhs , 2:4 )
  apifun_checklhs ( "distfun_normrnd" , lhs , 0:1 )
  //
  mu = varargin(1)
  sigma = varargin(2)
  //
  // Check type
  apifun_checktype ( "distfun_normrnd" , mu , "mu" , 1 , "constant" )
  apifun_checktype ( "distfun_normrnd" , sigma , "sigma" , 2 , "constant" )
  if ( rhs == 3 ) then
    v = varargin(3)
    apifun_checktype ( "distfun_normrnd" , v , "v" , 3 , "constant" )  
  end
  if ( rhs == 4 ) then
    m = varargin(3)
    n = varargin(4)
    apifun_checktype ( "distfun_normrnd" , m , "m" , 3 , "constant" )  
    apifun_checktype ( "distfun_normrnd" , n , "n" , 4 , "constant" )  
  end
  //
  // Check size
  if ( rhs == 3 ) then
    apifun_checkvector ( "distfun_normrnd" , v , "v" , 3 , 2 )
  end
  if ( rhs == 4 ) then
    apifun_checkscalar ( "distfun_normrnd" , m , "m" , 3 )
    apifun_checkscalar ( "distfun_normrnd" , n , "n" , 4 )
  end
  //
  // Check content
  tiny = number_properties("tiny")
  apifun_checkgreq ( "distfun_normrnd" , sigma , "sigma" , 2 , tiny )
  if ( rhs == 3 ) then
    apifun_checkflint ( "distfun_normrnd" , v , "v" , 3 )
    apifun_checkgreq ( "distfun_normrnd" , v , "v" , 3 , 1 )
  end
  if ( rhs == 4 ) then
    apifun_checkflint ( "distfun_normrnd" , m , "m" , 3 )
    apifun_checkgreq ( "distfun_normrnd" , m , "m" , 3 , 1 )
    apifun_checkflint ( "distfun_normrnd" , n , "n" , 4 )
    apifun_checkgreq ( "distfun_normrnd" , n , "n" , 4 , 1 )
  end
  //
  [mu,sigma] = apifun_expandfromsize ( 2 , varargin(1:$) )
  m = size(mu,"r")
  n = size(sigma,"c")
  //
  urnd = grand ( m , n , "unf" , 0 , 1 )
  R = distfun_norminv ( urnd , mu , sigma )
endfunction

