// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_unifrnd ( varargin )
    // Uniform random numbers
    //
    // Calling Sequence
    //   R = distfun_unifrnd ( a , b )
    //   R = distfun_unifrnd ( a , b , v )
    //   R = distfun_unifrnd ( a , b , m , n )
    //
    // Parameters
    //   a : a 1x1 or nxm matrix of doubles, the lower bound
    //   b : a 1x1 or nxm matrix of doubles, the upper bound
    //   v : a 1x2 or 2x1 matrix of doubles, the size of R
    //   v(1) : the number of rows of R
    //   v(2) : the number of columns of R
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers.
    //
    // Description
    //   Generates random variablesfrom the Uniform distribution function.
    //
    //   Any scalar input argument is expanded to a matrix of doubles 
    //   of the same size as the other input arguments.
    //
    //   As a side effect, it modifies the internal seed of the grand function.
    //
    // Examples
    //
    // // Set the seed so as to always get the same results.
    // grand("setsd",1);
    //
    // // Use R = distfun_unifrnd ( a , b )
    // distfun_unifrnd(1:6,2:7)
    // distfun_unifrnd(1:6,7)
    // distfun_unifrnd(1,2:7)
    //
    // // Check mean and variance for R = distfun_unifrnd ( a , b )
    // N = 1000;
    // a = 1:6;
    // b = 2:7;
    // m = (a+b)/2; // Expectation
    // v = (b-a).^2/12.; // Variance
    // for i = 1:N
    //   computed(i,1:6) = distfun_unifrnd(a,b);
    // end
    // c = mean(computed(1:N,1:6) , "r" )
    // disp(m)
    // c = st_deviation(computed(1:N,1:6) , "r" )
    // e = sqrt(v)
    //
    // // Check R = distfun_unifrnd ( a , b , v )
    // computed = distfun_unifrnd(2,3,[1 5])
    // computed = distfun_unifrnd(2,3,[3 2])
    //
    // // Check mean and variance for R = distfun_unifrnd ( a , b )
    // N = 1000;
    // a = 2;
    // b = 3;
    // m = (a+b)/2; // Expectation
    // v = (b-a).^2/12.; // Variance
    // computed = distfun_unifrnd(2,3,[1 N]);
    // c = mean(computed(1:N) )
	// disp(m)
    // c = st_deviation(computed(1:N) )
    // e = sqrt(v)
    //
    // // Check R = distfun_unifrnd ( a , b , m , n )
    // computed = distfun_unifrnd([1 2 3;4 5 6],7,2,3)
    // computed = distfun_unifrnd(4,5,2,3)
    // computed = distfun_unifrnd(0,[1 2 3;4 5 6],2,3)
    //
    // // Check mean and variance for R = distfun_unifrnd ( a , b )
    // N = 1000;
    // a = 2;
    // b = 3;
    // m = a .* b; // Expectation
    // v = m .* b; // Variance
    // computed = distfun_unifrnd(a,b,1,N);
    // c = mean(computed(1:N) )
	// disp(m)
    // c = st_deviation(computed(1:N) )
    // e = sqrt(v)
    //
    // // Make a plot of the actual distribution of the numbers
    // scf();
    // a = 2;
    // b = 3;
    // data = distfun_unifrnd(a,b,1,1000);
    // histplot(10,data)
    // x = linspace(a-1,b+1,1000);
    // y = distfun_unifpdf(x,a,b);
    // plot(x,y)
    //
    // Authors
    //   Copyright (C) 2012 - Michael Baudin

    [lhs,rhs]=argn()
    apifun_checkrhs ( "distfun_unifrnd" , rhs , 2:4 )
    apifun_checklhs ( "distfun_unifrnd" , lhs , 0:1 )
    //
    a = varargin(1)
    b = varargin(2)
    //
    // Check type
    apifun_checktype ( "distfun_unifrnd" , a , "a" , 1 , "constant" )
    apifun_checktype ( "distfun_unifrnd" , b , "b" , 2 , "constant" )
    if ( rhs == 3 ) then
        v = varargin(3)
        apifun_checktype ( "distfun_unifrnd" , v , "v" , 3 , "constant" )  
    end
    if ( rhs == 4 ) then
        m = varargin(3)
        n = varargin(4)
        apifun_checktype ( "distfun_unifrnd" , m , "m" , 3 , "constant" )  
        apifun_checktype ( "distfun_unifrnd" , n , "n" , 4 , "constant" )  
    end
    //
    // Check size
    if ( rhs == 3 ) then
        apifun_checkvector ( "distfun_unifrnd" , v , "v" , 3 , 2 )
    end
    if ( rhs == 4 ) then
        apifun_checkscalar ( "distfun_unifrnd" , m , "m" , 3 )
        apifun_checkscalar ( "distfun_unifrnd" , n , "n" , 4 )
    end
    //
    // Check content
    if ( rhs == 3 ) then
        apifun_checkflint ( "distfun_unifrnd" , v , "v" , 3 )
        apifun_checkgreq ( "distfun_unifrnd" , v , "v" , 3 , 1 )
    end
    if ( rhs == 4 ) then
        apifun_checkflint ( "distfun_unifrnd" , m , "m" , 3 )
        apifun_checkgreq ( "distfun_unifrnd" , m , "m" , 3 , 1 )
        apifun_checkflint ( "distfun_unifrnd" , n , "n" , 4 )
        apifun_checkgreq ( "distfun_unifrnd" , n , "n" , 4 , 1 )
    end
    //
    [a,b] = apifun_expandfromsize ( 2 , varargin(1:$) )
    apifun_checkgreq( "distfun_unifrnd" , b , "b" , 2 , a )
    //
    m = size(a,"r")
    n = size(a,"c")
    urnd = grand ( m , n , "unf" , 0 , 1 )
    R = distfun_unifinv ( urnd , a , b )
endfunction

